/* ***********************************************************************
**
**	lcd.c
**
**  Copyright (C) 2008  Frank Hnefeld
**
**  DEM 240320A TMH-PW-N Display (Display-Elektronik.de)
**	 mit HX8346-A Controller
**   an ATmega128 8Mhz bei 3,3V
**
**
**
*********************************************************************** */

#include "lcd.h"

#define NOP() asm volatile ("nop" ::)

void mswait(uint16_t ms)
{
	uint16_t del;
	while(ms--)
	{
		for(del = 0; del < 1000; del++)
		{NOP();}
	}
}

void lcd_init(void)
{
	uint16_t k;
	port_init_io();


	LCD_DATA_LB_PORT = 0x22;
	LCD_CTR_PORT &= ~(1<<LCD_CS);	//Chip Select
	mswait(1);
	LCD_CTR_PORT &= ~(1<<LCD_RS);	//Index-Register
	mswait(1);
	LCD_CTR_PORT &= ~(1<<LCD_WR);	//Schreiben
	mswait(1);
	LCD_CTR_PORT |= (1<<LCD_WR);
	mswait(1);
	LCD_CTR_PORT |= (1<<LCD_RS);
	LCD_DATA_LB_PORT = 0x5A;
	LCD_DATA_HB_PORT = 0x5A;

	mswait(1);
	for(k=0;k<57600;k++)
	{
		//Einschreiben in den Grafikspeicher
		LCD_CTR_PORT &= ~(1<<LCD_WR);	//Schreiben;
		NOP();
		NOP();
		LCD_CTR_PORT |= (1<<LCD_WR);
		NOP();
		//Display on Step 3
		NOP();
	}
	LCD_CTR_PORT |= (1<<LCD_CS);
}


void port_init_io(void)
{
	LCD_CTR_PORT |= (1<<LCD_RESET)|(1<<LCD_CS)|(1<<LCD_RD)|(1<<LCD_WR)|(1<<LCD_RS);
	LCD_CTR_DIR |= (1<<LCD_RESET)|(1<<LCD_CS)|(1<<LCD_RD)|(1<<LCD_WR)|(1<<LCD_RS);

	LCD_DATA_LB_PORT = 0xFF;
	LCD_DATA_LB_DIR = 0xFF;
	LCD_DATA_HB_PORT = 0xFF;
	LCD_DATA_HB_DIR = 0xFF;

	LCD_CTR_PORT &= ~(1<<LCD_RESET);	//Poweron Reset
	mswait(1);
	LCD_CTR_PORT |= (1<<LCD_RESET);
	mswait(100);

	lcd_wrcmd (0x30, 0x80);	//Display on Step 1
	lcd_wrcmd (0x26, 0x04);	//Display on Step 2
	mswait(100);			//Display on Step 3
	lcd_wrcmd (0x26, 0x24);	//Display on Step 4
	lcd_wrcmd (0x26, 0x2B);	//Display on Step 5
	mswait(100);			//Display on Step 6
	lcd_wrcmd (0x26, 0x3B);	//Display on Step 7

	lcd_wrcmd (0x02, 0x00);	//Write Column Adress Start Register Upper Byte
	lcd_wrcmd (0x03, 0x00);	//Write Column Adress Start Register Lower Byte
	lcd_wrcmd (0x04, 0x00);	//Write Column Adress End Register Upper Byte
	lcd_wrcmd (0x05, 0xF0);	//Write Column Adress End Register Lower Byte
	lcd_wrcmd (0x06, 0x00);	//Write Row Adress Start Register Upper Byte
	lcd_wrcmd (0x07, 0x00);	//Write Row Adress Start Register Lower Byte
	lcd_wrcmd (0x08, 0x00);	//Write Row Adress End Register Upper Byte
	lcd_wrcmd (0x09, 0xF0);	//Write Row Adress End Register Lower Byte
}


void lcd_wrcmd(char LcdReg ,char data)
{
	LCD_DATA_LB_PORT = LcdReg;
	NOP();
	LCD_CTR_PORT &= ~(1<<LCD_CS);	//Chip Select
	NOP();
	LCD_CTR_PORT &= ~(1<<LCD_RS);	//Index-Register
	NOP();							// zwischen RS und Write 10 ns
	LCD_CTR_PORT &= ~(1<<LCD_WR);	//Schreiben
	NOP();							//Writepulslnge min. 35 ns
	LCD_CTR_PORT |= (1<<LCD_WR);
	NOP();							// nach den Write 10 ns bevor RS _|-
	LCD_CTR_PORT |= (1<<LCD_RS);
	NOP();
	LCD_DATA_LB_PORT = data;
	NOP();
	LCD_CTR_PORT &= ~(1<<LCD_WR);	//Schreiben
	NOP();
	LCD_CTR_PORT |= (1<<LCD_WR);
	NOP();
	LCD_CTR_PORT |= (1<<LCD_CS);
	NOP();

}

